# the plugin code was generated by this script
# you should not change the plugin code directly, but this script
# note: this script only creates objects in your workspace,
# *EXCEPT* for the last call, see below.

local({
# set the output directory to overwrite the actual plugin
output.dir <- tempdir()
overwrite <- TRUE

require(rkwarddev)

about.info <- rk.XML.about(
	name="rk.FactorAnalysis",
	author=c(
		person(given="Meik", family="Michalke",
			email="meik.michalke@hhu.de", role=c("aut","cre"))),
	about=list(desc="RKWard GUI to conduct principal component and factor analysis (using the psych package).",
		version="0.01-0", url="http://rkward.sf.net"),
	dependencies=list(rkward.min="0.5.6"),
	package=list(c(name="psych"))
	)

#############
## principal component & factor analysis
#############
# using principal()/fa() of the package 'psych'

radio.analysis <- rk.XML.radio("Factoring method", options=list(
		"Principal component analysis"=c(val="PCA"),
		"Explorative factor analysis"=c(val="EFA", chk=TRUE)
	))
var.select <- rk.XML.varselector(label="Select data")
var.data <- rk.XML.varslot(label="Correlation matrix (or raw data matrix)", source=var.select, required=TRUE)

save.results <- rk.XML.saveobj("Save results to workspace", initial="analysis.results")

drp.rotation.PCA <- rk.XML.dropdown("Rotation method", options=list(
		"none"=c(val="none"),
		"varimax (orthogonal)"=c(val="varimax", chk=TRUE),
		"quatimax (orthogonal)"=c(val="quatimax"),
		"promax"=c(val="promax"),
		"oblimin"=c(val="oblimin"),
		"simplimax"=c(val="simplimax"),
		"cluster"=c(val="cluster")
	), id.name="drp_PCA_rotate")
drp.rotation.EFA <- rk.XML.dropdown("Rotation method", options=list(
		"none"=c(val="none"),
		"varimax (orthogonal)"=c(val="varimax"),
		"quatimax (orthogonal)"=c(val="quatimax"),
		"bentlerT (orthogonal)"=c(val="bentlerT"),
		"geominT (orthogonal)"=c(val="geominT"),
		"bifactor (orthogonal)"=c(val="bifactor"),
		"promax"=c(val="promax"),
		"oblimin"=c(val="oblimin", chk=TRUE),
		"simplimax"=c(val="simplimax"),
		"bentlerQ"=c(val="bentlerQ"),
		"geominQ"=c(val="geominQ"),
		"biquartimin"=c(val="biquartimin"),
		"cluster"=c(val="cluster")
	), id.name="drp_EFA_rotate")

# common options
spin.nfactors <- rk.XML.spinbox("Number of factors to extract", min=1, initial=1, real=FALSE)

chk.residuals <- rk.XML.cbox("Show residuals", value="true")
radio.missings <- rk.XML.dropdown("Dealing with missing values", options=list(
	"Don't impute missing values"=c(val="none", chk=TRUE),
	"Replace with median"=c(val="median"),
	"Replace with mean"=c(val="mean")))
spin.nobs <- rk.XML.spinbox("Number of observations to find the correlation matrix (GOF statistics)", min=0, initial=0, real=FALSE)
	
tab1.data <- rk.XML.row(
		var.select,
		rk.XML.col(
			rk.XML.frame(radio.analysis),
			rk.XML.frame(var.data),
			spin.nfactors,
			rk.XML.stretch(),
			save.results
		)
	)

 tab2.options <- rk.XML.row(
 		rk.XML.col(
 			rk.XML.frame(drp.rotation.PCA),
 			rk.XML.frame(drp.rotation.EFA),
 			rk.XML.frame(
				rk.XML.row(
					rk.XML.col(chk.residuals),
					rk.XML.col()
				)
			),
			chk.scores <- rk.XML.frame(radio.missings, label="Find component scores", checkable=TRUE, chk=FALSE),
 			spin.nobs,
 			rk.XML.stretch()
 		)
 	)

full.dialog <- rk.XML.dialog(rk.XML.tabbook(label="Factor analysis",
		tabs=list("Data"=tab1.data, "Options"=tab2.options)
	), label="Factor analysis")

## logic section
	lgc.sect <- rk.XML.logic(
		FA.gov.between <- rk.XML.convert(sources=list(string=radio.analysis), mode=c(equals="PCA")),
		rk.XML.connect(governor=FA.gov.between, client=drp.rotation.PCA, set="visible"),
		rk.XML.connect(governor=FA.gov.between, client=drp.rotation.EFA, set="visible", not=TRUE)
)

## JavaScript
# discard this objest later, we just need the name...
js.frm.score <- rk.JS.vars(chk.scores, modifiers="checked")
js.calc <- rk.paste.JS(
	echo("analysis.results <- "),
	ite(id(radio.analysis, " == \"PCA\""),
		echo("principal("),
		echo("fa(")),
	ite(var.data, echo("r=", var.data)),
	ite(id(spin.nfactors, " > 1"), echo(",\n\tnfactors=", spin.nfactors)),
	ite(chk.residuals, echo(",\n\tresiduals=TRUE")),
	ite(id(radio.analysis, " == \"PCA\""),
		echo(",\n\trotate=\"", drp.rotation.PCA, "\""),
		echo(",\n\trotate=\"", drp.rotation.EFA, "\"")),
	ite(id(spin.nobs, " > 0"), echo(",\n\tn.obs=", spin.nobs)),
	ite(js.frm.score, echo(",\n\tscores=TRUE")),
	ite(id(js.frm.score, " & ", radio.missings, " != \"none\""), echo(",\n\tmissing=TRUE,\n\timpute=\"", radio.missings, "\"")),
	echo(")\n\n")
)

js.print <- rk.paste.JS(
 	echo("rk.print(analysis.results[[\"loadings\"]], cutoff=NULL)\n")
)


#############
## if you run the following function call, files will be written to tempdir!
#############
# this is where it get's serious, that is, here all of the above is put together into one plugin

rk.FactorAnalysis.dir <<- rk.plugin.skeleton(
	about.info,
	path=output.dir,
	xml=list(
		logic=lgc.sect,
		dialog=full.dialog),
	js=list(
		require="psych",
		calculate=js.calc,
		printout=js.print),
	pluginmap=list(name="Factor analysis", hierarchy=list("analysis", "Factor analysis")),
#	components=list(),
	create=c("pmap", "xml", "js", "desc"),
	overwrite=overwrite,
	tests=FALSE,
#	edit=TRUE,
	load=TRUE,
	show=TRUE)
})
